﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Dapfor.Wpf.Controls;

namespace Dapfor.Wpf.Grid.Demo.Examples.Basics.CustomFilters
{
    /// <summary>
    ///     Interaction logic for SalesFilter.xaml
    /// </summary>
    public partial class SalesFilter : UserControl
    {
        private bool _suspendUpdates;

        public SalesFilter()
        {
            InitializeComponent();
        }

        private void OnSliderValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (!_suspendUpdates && Column != null)
            {
                Column.Tag = e.NewValue;
                Column.Filter = new FilterRules(Column, e.NewValue);
            }
        }

        protected override void OnPreviewKeyDown(KeyEventArgs e)
        {
            base.OnPreviewKeyDown(e);
            if (e.Key == Key.Escape)
            {
                Column.CloseFilter();
            }
        }

        private class FilterRules : IFilter
        {
            private readonly Column _column;
            private readonly double _value;

            public FilterRules(Column column, double value)
            {
                _column = column;
                _value = value;
            }

            public bool IsFiltered(Row row)
            {
                var cell = row[_column.Id];
                var v = cell != null ? cell.Value : null;
                return (v as double?) > _value;
            }
        }

        #region ColumnProperty

        /// <summary>
        ///     Identifies the <see cref="Column" /> dependency property.
        /// </summary>
        public static readonly DependencyProperty ColumnProperty = DependencyProperty.Register("Column", typeof (Column), typeof (SalesFilter), new FrameworkPropertyMetadata(null, OnChangeColumn));

        private static void OnChangeColumn(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            //Restore persisted data
            var control = (SalesFilter) d;
            if (control.Column != null && control.Column.Tag is double)
            {
                control._suspendUpdates = true;
                control.Slider.Value = (double) control.Column.Tag;
                control._suspendUpdates = false;
            }
        }

        /// <summary>
        ///     The actually filtering <see cref="Column" />.
        /// </summary>
        public Column Column
        {
            get { return (Column) GetValue(ColumnProperty); }
            set { SetValue(ColumnProperty, value); }
        }

        #endregion
    }
}