﻿using System;
using System.Security.AccessControl;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Demo.Internal;
using Microsoft.Win32;

namespace Dapfor.Wpf.Grid.Demo.Examples.Miscellaneous.RegistryExplorer
{
    /// <summary>
    /// Interaction logic for RegistryExplorerDemo.xaml
    /// </summary>
    [Example("Miscellaneous", "Registry explorer", "Description_RegistryExplorerDemo")]
    public partial class RegistryExplorerDemo : IDisposable
    {
        public RegistryExplorerDemo()
        {
            InitializeComponent();

            gridKeys.Headers[0]["Name"].ValueConverter = new RegistryKeyNameConverter();

            gridKeys.Rows.Add(Registry.ClassesRoot);
            gridKeys.Rows.Add(Registry.CurrentUser);
            Row row = gridKeys.Rows.Add(Registry.LocalMachine);
            row.Expanded = true;
            Row firstChildRow = row.FirstVisibleChild;
            if (firstChildRow != null)
            {
                firstChildRow.Selected = true;
                firstChildRow.Focused = true;
            }

            gridKeys.Rows.Add(Registry.Users);
            gridKeys.Rows.Add(Registry.CurrentConfig);
        }

        private void OnGridRowExpansionChanged(object sender, GridControlRowEventArgs e)
        {
            //When the row is opening, we can add subkeys to this row
            var key = e.Row != null ? e.Row.DataObject as RegistryKey : null;
            if (key != null && e.Row.Children.Count == 0)
            {
                foreach (var subKeyName in key.GetSubKeyNames())
                {
                    try
                    {
                        e.Row.Add(key.OpenSubKey(subKeyName, RegistryKeyPermissionCheck.ReadSubTree));
                    }
                    catch (Exception)
                    {
                    }
                    

                    
                }
            }
        }

        private void OnGridExpansionButtonState(object sender, GridControlExpansionButtonStateEventArgs e)
        {
            //When a RegistryKey is not opened yet, we can display an expansion icon (+) if it contains some subkeys
            if (e.State == ExpansionButtonState.Hidden)
            {
                var key = (RegistryKey)e.Row.DataObject;
                if (key != null && key.SubKeyCount > 0)
                {
                    e.State = e.Row.Expanded ? ExpansionButtonState.Collapsed : ExpansionButtonState.Expanded;
                }
            }
        }

        private void OnGridRowRemoved(object sender, GridControlRowEventArgs e)
        {
            //Dispose all already created keys except HKLM, HKCU etc.
            if (e.Row.Level > 0)
            {
                RegistryKey registryKey = e.Row.DataObject as RegistryKey;
                if (registryKey != null)
                {
                    registryKey.Dispose();
                }
            }
            
        }

        private void OnFocusedKeyChanged(object sender, GridControlFocusedRowEventArgs e)
        {
            //Populate the second grid if the focus is changed
            var key = e.NewFocusedRow != null ? (RegistryKey)e.NewFocusedRow.DataObject : null;
            gridValues.Rows.Clear();
            try
            {
                if (key != null)
                {
                    string[] values = key.GetValueNames();
                    foreach (var valueName in values)
                    {
                        RegistryValueKind valueKind = key.GetValueKind(valueName);
                        object valueData = key.GetValue(valueName);
                        gridValues.Rows.Add(new RegistryValyeType(valueName, valueKind, valueData));
                    }
                }
            }
            catch (Exception)
            {
                //Access is denied to the RegisteryKey
            }
        }


        public void Dispose()
        {
            gridKeys.Rows.Clear();
            gridValues.Rows.Clear();
        }
    }
}