﻿using System;
using System.Windows.Threading;
using Dapfor.Wpf.Grid.Demo.Internal;

namespace Dapfor.Wpf.Grid.Demo.Examples.Performance.RealTimeUpdates
{
    /// <summary>
    ///     Interaction logic for RealTimeUpdatesDemo.xaml
    /// </summary>
    [Example("Performance", "Realtime updates", "Description_RealTimeUpdatesDemo")]
    public partial class RealTimeUpdatesDemo : IDisposable
    {
        private readonly DataObjectCollection _collection;
        private readonly Random _random = new Random();
        private readonly DispatcherTimer _timer = new DispatcherTimer();
        private DateTime _lastStatStamp = DateTime.UtcNow;
        private long _updatesCount;

        public RealTimeUpdatesDemo()
        {
            InitializeComponent();

            _collection = (DataObjectCollection) FindResource("collection");

            _timer.Interval = TimeSpan.FromMilliseconds(1);
            _timer.Tick += OnTick;
            _timer.Start();
        }

        public void Dispose()
        {
            _timer.Stop();
            if (_collection != null)
            {
                _collection.Clear();
            }
        }


        private void OnTick(object sender, EventArgs e)
        {
            if (_collection != null && _collection.Count > 0)
            {
                var value = 3*(int) slider.Value + 1;

                for (var i = 0; i < value; i++)
                {
                    var obj = _collection[_random.Next(_collection.Count)];
                    obj.RandomUpdate();
                    _updatesCount++;
                }
            }

            if (DateTime.UtcNow - _lastStatStamp > TimeSpan.FromSeconds(1))
            {
                labelUpdates.Text = string.Format("Updates: {0}/sec", _updatesCount);
                _updatesCount = 0;
                _lastStatStamp = DateTime.UtcNow;
            }
        }
    }
}