﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Media;
using Dapfor.Wpf.Grid.Demo.Internal;


namespace Dapfor.Wpf.Grid.Demo.Examples.WhyUs.ObservableCollectionsInRows
{
    

    /// <summary>
    ///     Interaction logic for ObservableCollectionsInRowsDemo.xaml
    /// </summary>
    [Example("Why us?", "Observable collections in rows", "Description_WhyUsObservableCollectionsInRows")]
    public partial class ObservableCollectionsInRowsDemo
    {
        readonly Random _random = new Random();
        readonly IList<RootClass> _collection = new ObservableCollection<RootClass>();
        private int _index;

        public ObservableCollectionsInRowsDemo()
        {
            InitializeComponent();

            //Build collection
            
            for (int i = 0; i < 4; ++i)
            {
                //Populate with top-level items
                var rootObject = new RootClass(_index++,
                    _random.NextDouble(),
                    _random.Next(1000, 20000),
                    _random.Next() % 2 == 0 ? RootClass.EnumExample.Internal : RootClass.EnumExample.External);

                //Add ten children to each root
                for (var j = 0; j < _random.Next(3, 6); ++j)
                {
                    var childObject = new ChildClass(i,
                        j,
                        DateTime.Now + new TimeSpan(1000000000000*(_random.Next()%1000)),
                        _random.Next(5),
                        new TimeSpan(_random.Next()));
                    //Add as a child

                    rootObject.SomeChildren.Add(childObject);
                }
                _collection.Add(rootObject);
            }

            //Bind grid to the collection
            grid.ItemsSource = _collection;


            LinearGradientBrush brush = new LinearGradientBrush();
            brush.StartPoint = new Point(0, 0);
            brush.EndPoint = new Point(0, 1);
            brush.GradientStops.Add(new GradientStop(Colors.Transparent, 0));
            brush.GradientStops.Add(new GradientStop(Color.FromArgb(60, 255, 0, 0), .1));
            brush.GradientStops.Add(new GradientStop(Color.FromArgb(100, 255, 155, 0), .5));
            brush.GradientStops.Add(new GradientStop(Color.FromArgb(60, 255, 0, 0), .9));
            brush.GradientStops.Add(new GradientStop(Colors.Transparent, 1));

            grid.RowAdded += (sender, e) =>
            {
                e.Row.EnsureVisible();
                foreach (var column in e.Row.Header.Columns)
                {
                    e.Row[column.Id].Blink(TimeSpan.FromSeconds(1), brush);
                }
            };
        }

        private void OnAddRoot(object sender, RoutedEventArgs e)
        {
            var rootObject = new RootClass(_index++,
                   _random.NextDouble(),
                   _random.Next(1000, 20000),
                   _random.Next() % 2 == 0 ? RootClass.EnumExample.Internal : RootClass.EnumExample.External);

            _collection.Add(rootObject);

        }

        private void OnRemoveRoot(object sender, RoutedEventArgs e)
        {
            if (_collection.Count > 0)
            {
                _collection.RemoveAt(_random.Next(_collection.Count));    
            }
        }

        private void OnAddChild(object sender, System.Windows.RoutedEventArgs e)
        {
            if (_collection.Count > 0)
            {
                RootClass root = _collection[_random.Next(_collection.Count)];

                var childObject = new ChildClass(root.Id,
                       root.SomeChildren.Count,
                       DateTime.Now + new TimeSpan(1000000000000 * (_random.Next() % 1000)),
                       _random.Next(5),
                       new TimeSpan(_random.Next()));
                
                root.SomeChildren.Add(childObject);
            }

        }

        private void OnRemoveChild(object sender, System.Windows.RoutedEventArgs e)
        {
            if (_collection.Count > 0)
            {
                RootClass root = _collection[_random.Next(_collection.Count)];
                if (root.SomeChildren.Count == 0)
                {
                    root = null;
                    foreach (var node in grid.Nodes)
                    {
                        RootClass o = (RootClass)node.DataObject;
                        if (o.SomeChildren.Count > 0)
                        {
                            root = o;
                            break;
                        }

                    }
                }

                if (root != null && root.SomeChildren.Count > 0)
                {
                    root.SomeChildren.RemoveAt(_random.Next(root.SomeChildren.Count));   
                }
            }
        }
    }
}
